# SPDX-License-Identifier: MIT
# Copyright (c) 2025 Matthias Bilger <matthias@bilger.info>
import argparse  # Added import for argparse
import asyncio
import logging
import pathlib
import signal
import threading

from mutenix.macropad import Macropad
from mutenix.tray_icon import run_trayicon
from mutenix.updates import check_for_self_update
from mutenix.version import MAJOR
from mutenix.version import MINOR
from mutenix.version import PATCH

# Configure logging to write to a file
log_file_path = pathlib.Path.cwd() / "mutenix.log"
logging.basicConfig(
    level=logging.DEBUG,
    filename=log_file_path,
    filemode="a",
    format="%(asctime)s - %(name)-25s [%(levelname)-8s]: %(message)s",
)
_logger = logging.getLogger(__name__)


def parse_arguments():
    parser = argparse.ArgumentParser(description="Mutenix Macropad Controller")
    parser.add_argument(
        "--config",
        type=pathlib.Path,
        help="Path to the configuration file",
    )
    parser.add_argument(
        "--update-file",
        type=str,
        help="Path to the update tar.gz file",
    )
    parser.add_argument(
        "--list-devices",
        action="store_true",
        help="List all connected devices",
    )
    return parser.parse_args()


def main(args: argparse.Namespace):
    check_for_self_update(MAJOR, MINOR, PATCH)

    def signal_handler(signal, frame):
        print("Shuting down...")
        _logger.info("SIGINT received, shutting down...")
        asyncio.create_task(macropad.stop())

    if args.list_devices:
        import hid

        for device in hid.enumerate():
            print(device)
        return

    signal.signal(signal.SIGINT, signal_handler)
    macropad = Macropad(args.config)

    if args.update_file:
        _logger.info("Starting manual update with file: %s", args.update_file)
        asyncio.run(macropad.manual_update(args.update_file))
        return

    def run_asyncio_loop():
        asyncio.run(macropad.process())

    loop_thread = threading.Thread(target=run_asyncio_loop)
    loop_thread.start()

    run_trayicon(macropad)

    loop_thread.join()


def runmain():
    args = parse_arguments()
    logging.basicConfig(level=logging.INFO)
    main(args)


if __name__ == "__main__":
    runmain()



# SPDX-License-Identifier: MIT
# Copyright (c) 2025 Matthias Bilger matthias@bilger.info
from __future__ import annotations

from pathlib import Path
from unittest.mock import mock_open
from unittest.mock import patch

import yaml
from mutenix.config import Config
from mutenix.config import CONFIG_FILENAME
from mutenix.config import create_default_config
from mutenix.config import find_config_file
from mutenix.config import load_config


def test_find_config_file_default_location():
    with patch("pathlib.Path.exists", return_value=True):
        config_path = find_config_file()
        assert config_path == Path(CONFIG_FILENAME)


def test_find_config_file_not_found():
    with patch("pathlib.Path.exists", return_value=False):
        config_path = find_config_file()
        assert config_path == Path(CONFIG_FILENAME)


def test_load_config_default():
    with patch("pathlib.Path.exists", return_value=False):
        with patch("builtins.open", mock_open(read_data="")):
            with patch(
                "mutenix.config.create_default_config",
            ) as mock_create_default_config:
                default_config = create_default_config()
                default_config.file_path = str(Path(CONFIG_FILENAME))
                mock_create_default_config.return_value = default_config
                config = load_config()
                assert config == mock_create_default_config.return_value


def test_load_config_file_not_found():
    with patch("pathlib.Path.exists", return_value=False):
        with patch("builtins.open", mock_open(read_data="")):
            with patch(
                "mutenix.config.create_default_config",
            ) as mock_create_default_config:
                mock_create_default_config.return_value = create_default_config()
                with patch("mutenix.config.save_config") as mock_save_config:
                    config = load_config()
                    assert config == mock_create_default_config.return_value
                    mock_save_config.assert_called_once()


def test_load_config_yaml_error():
    with patch("pathlib.Path.exists", return_value=True):
        with patch("builtins.open", mock_open(read_data="invalid_yaml")):
            with patch("yaml.safe_load", side_effect=yaml.YAMLError):
                with patch(
                    "mutenix.config.create_default_config",
                ) as mock_create_default_config:
                    mock_create_default_config.return_value = create_default_config()
                    with patch("mutenix.config.save_config") as mock_save_config:
                        config = load_config()
                        assert config == mock_create_default_config.return_value
                        mock_save_config.assert_called_once()


def test_load_config_success():
    config_data = {
        "actions": [
            {"button_id": 1, "action": "toggle-mute"},
            {"button_id": 2, "action": "toggle-hand"},
        ],
        "double_tap_action": [],
        "teams_token": None,
    }
    with patch("pathlib.Path.exists", return_value=True):
        with patch("builtins.open", mock_open(read_data=yaml.dump(config_data))):
            with patch("yaml.safe_load", return_value=config_data):
                config = load_config()
                assert config == Config(
                    **config_data,
                )



# SPDX-License-Identifier: MIT
# Copyright (c) 2025 Matthias Bilger matthias@bilger.info
from __future__ import annotations

import argparse
from pathlib import Path
from unittest.mock import patch

import pytest
from mutenix.__main__ import main
from mutenix.__main__ import parse_arguments


def test_parse_arguments_no_update_file():
    test_args = []
    with patch("sys.argv", ["__main__.py"] + test_args):
        args = parse_arguments()
        assert args.update_file is None


def test_parse_arguments_with_update_file():
    test_args = ["--update-file", "path/to/update.tar.gz"]
    with patch("sys.argv", ["__main__.py"] + test_args):
        args = parse_arguments()
        assert args.update_file == "path/to/update.tar.gz"


def test_parse_arguments_list_devices():
    test_args = ["--list-devices"]
    with patch("sys.argv", ["__main__.py"] + test_args):
        args = parse_arguments()
        assert args.list_devices is True


def test_parse_arguments_config():
    test_args = ["--config", "path/to/config.yaml"]
    with patch("sys.argv", ["__main__.py"] + test_args):
        args = parse_arguments()
        assert args.config == Path("path/to/config.yaml")


@pytest.fixture
def mock_macropad():
    with patch("mutenix.__main__.Macropad", autospec=True) as MockMacropad:
        yield MockMacropad.return_value


@pytest.fixture
def mock_check_for_self_update():
    with patch("mutenix.__main__.check_for_self_update") as mock_update:
        yield mock_update


@pytest.fixture
def mock_run_trayicon():
    with patch("mutenix.__main__.run_trayicon") as mock_trayicon:
        yield mock_trayicon


@pytest.fixture
def mock_signal():
    with patch("signal.signal") as mock_signal:
        yield mock_signal


def test_main_no_update_file(
    mock_macropad,
    mock_check_for_self_update,
    mock_run_trayicon,
    mock_signal,
):
    args = argparse.Namespace(update_file=None, list_devices=False, config=None)

    with (
        patch("threading.Thread.start", autospec=True) as mock_thread_start,
        patch("threading.Thread.join", autospec=True) as mock_thread_join,
        patch("asyncio.run", autospec=True),
    ):
        main(args)

        mock_check_for_self_update.assert_called_once()
        mock_signal.assert_called_once()
        mock_thread_start.assert_called_once()
        mock_run_trayicon.assert_called_once_with(mock_macropad)
        mock_thread_join.assert_called_once()


def test_main_with_update_file(mock_macropad, mock_check_for_self_update, mock_signal):
    args = argparse.Namespace(
        update_file="path/to/update.tar.gz",
        list_devices=False,
        config=None,
    )

    with patch("asyncio.run", autospec=True) as mock_asyncio_run:
        main(args)

        mock_check_for_self_update.assert_called_once()
        mock_signal.assert_called_once()
        mock_asyncio_run.assert_called_once()



# SPDX-License-Identifier: MIT
# Copyright (c) 2025 Matthias Bilger <matthias@bilger.info>
import asyncio
import webbrowser
from pathlib import Path
from typing import Any

from mutenix.macropad import Macropad
from PIL import Image

my_icon: Any | None = None


def load_image(file_name):
    file_path = Path(__file__).parent / "assets" / file_name
    return Image.open(file_path)


def run_trayicon(macropad: Macropad):  # pragma: no cover
    global my_icon
    from pystray import Icon as icon, Menu as menu, MenuItem as item

    def open_url(endpoint=""):
        def open_url(icon, item):
            address = (
                macropad.virtual_keypad_address
                if macropad.virtual_keypad_address != "0.0.0.0"
                else "127.0.0.1"
            )
            webbrowser.open(
                f"http://{address}:{macropad.virtual_keypad_port}{endpoint}",
            )

        return open_url

    def activate_serial_console(icon, item):
        macropad.activate_serial_console()

    def deactivate_serial_console(icon, item):
        macropad.deactivate_serial_console()

    def activate_filesystem(icon, item):
        macropad.activate_filesystem()

    def quit_macropad(icon, item):
        asyncio.run(macropad.stop())
        icon.stop()

    def nothing(icon, item):
        pass

    my_icon = icon(
        "MUTENIX",
        load_image("icon_all_red_64.png"),
        menu=menu(
            item(
                "Open Virtual Macropad",
                open_url("/"),
            ),
            item(
                "Teams connected",
                nothing,
                checked=lambda x: macropad.teams_connected,
                enabled=False,
            ),
            item(
                "Device connected",
                nothing,
                checked=lambda x: macropad.device_connected,
                enabled=False,
            ),
            item(
                "Help",
                open_url("/help"),
            ),
            item(
                "About",
                open_url("/about"),
            ),
            item(
                "Debug Options",
                menu(
                    item("Activate Serial Console", activate_serial_console),
                    item("Deactivate Serial Console", deactivate_serial_console),
                    item("Activate Filesystem", activate_filesystem),
                ),
            ),
            item(
                "Quit",
                quit_macropad,
            ),
        ),
    )
    my_icon.run()
